# Contains methods for setting up IHT ModVars

import numpy as np

import AvenirCommon.Util as acu

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.HW.HWConst as hwc
import SpectrumCommon.Const.HW.HWModVarFields as hwmvf
import SpectrumCommon.Util.HW.HWUtil as hwu
import SpectrumCommon.Modvars.HW.HWModVarUtil as hwmvu

#######################################################################################################
#                                                                                                     #
#                                       Staff types                                                   #
#                                                                                                     #
#######################################################################################################

def HW_create_staff_type_rec(ID = str, name = str, string_const = str, active = bool, staff_division_mstID = int, static_staff_type_map_mst_ID = int,
    num_facility_types = int, num_years = int, costing_mode = int):

    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        avg_days_worked_per_year = 240
        hours_in_a_work_day = 6
        annual_real_salary_incr = 3.1

    else:
        avg_days_worked_per_year = 220
        hours_in_a_work_day = 8
        annual_real_salary_incr = 0.0

    return {
        hwmvf.HW_STAFF_ID                                          : ID,
        hwmvf.HW_STAFF_STR_CONSTANT                                : string_const,
        hwmvf.HW_STAFF_NAME                                        : name,
        hwmvf.HW_STAFF_ACTIVE                                      : active,
        hwmvf.HW_STAFF_CUSTOM                                      : False,
        hwmvf.HW_STAFF_DIVISION_MST_ID                             : staff_division_mstID,
        hwmvf.HW_STAFF_STATIC_MAP_MST_ID                           : static_staff_type_map_mst_ID,
        hwmvf.HW_STAFF_BASE_YEAR                                   : np.full((hwc.HW_NUM_STAFFING_LEVELS), 0).tolist(),
        hwmvf.HW_STAFF_ANNUAL_SALARY                               : 0.0, #1000.0,                
        hwmvf.HW_STAFF_ANNUAL_REAL_SALARY_INCR                     : annual_real_salary_incr,
        hwmvf.HW_STAFF_ANNUAL_BENEFITS                             : 0.0,          
        hwmvf.HW_STAFF_ANNUAL_BENEFITS_ADJUST                      : 0,
        hwmvf.HW_STAFF_PERC_TIME_SPENT_NOT_TRAIN_SERV              : 0,
        hwmvf.HW_STAFF_AVG_DAYS_WORKED_PER_YEAR                    : avg_days_worked_per_year,
        hwmvf.HW_STAFF_HOURS_IN_WORK_DAY                           : hours_in_a_work_day,
        hwmvf.HW_STAFF_CURR_ATTRIT_PER_YEAR                        : [],
        hwmvf.HW_STAFF_PERC_REC_ATTRIT_REDUC_INCENTS               : [],
        hwmvf.HW_STAFF_ANNUAL_COST_ATTRIT_REDUC_INCENTS            : [],
        hwmvf.HW_STAFF_PRESERV_NUM_SCHOOLS                         : 0,
        hwmvf.HW_STAFF_PRESERV_YEARS_TRAINING                      : 0,
        hwmvf.HW_STAFF_PRESERV_ANNUAL_COST_PER_STUDENT             : 0.0,
        hwmvf.HW_STAFF_PRESERV_CURR_ENROLLMENT                     : 0,
        hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR              : 0,
        hwmvf.HW_STAFF_PRESERV_INST_NAMES                          : np.full((hwc.HW_NUM_PRESERV_TRAIN_INSTITUTIONS), '').tolist(),          
        hwmvf.HW_STAFF_PRESERV_AVG_YEARS_TRAINING_BY_INST          : np.full((hwc.HW_NUM_PRESERV_TRAIN_INSTITUTIONS), 0).tolist(),        
        hwmvf.HW_STAFF_PRESERV_ANNUAL_AVG_COST_PER_STUDENT_BY_INST : np.full((hwc.HW_NUM_PRESERV_TRAIN_INSTITUTIONS), 0.0).tolist(),    
        hwmvf.HW_STAFF_PRESERV_CURR_INTAKE_BY_INST                 : np.full((hwc.HW_NUM_PRESERV_TRAIN_INSTITUTIONS), 0).tolist(),  
        hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR_BY_INST      : np.full((hwc.HW_NUM_PRESERV_TRAIN_INSTITUTIONS), 0).tolist(),           
        hwmvf.HW_STAFF_TOTAL_ANNUAL_RECRUITMENT                    : 0,
        hwmvf.HW_STAFF_PERC_NEW_GRADS_PUBLIC_SECTOR                : 0.0,
        hwmvf.HW_STAFF_INSERV_TRAIN_COST_PERC_SALARY               : 0.0,
        hwmvf.HW_STAFF_TARGET_SETTING_METHOD                       : np.full((hwc.HW_NUM_STAFFING_LEVELS), hwc.HW_FAC_STAFF_STANDARDS_MST_ID).tolist(),
        hwmvf.HW_STAFF_TARGET_STAFF_POP_RATIO                      : np.full((hwc.HW_NUM_STAFFING_LEVELS), 0.0).tolist(),
        hwmvf.HW_STAFF_TARGET_STAFF_PER_FACILITY                   : np.full((hwc.HW_NUM_STAFFING_LEVELS, num_facility_types), 0).tolist(),
        hwmvf.HW_STAFF_TARGET_SCALE_UP_EXISTING_PLANS              : np.full((hwc.HW_NUM_STAFFING_LEVELS, num_years), 0).tolist(),
        hwmvf.HW_STAFF_TARGET_ENROLLMENT                           : np.full((num_years), 0).tolist(),          
        hwmvf.HW_STAFF_TARGET_GRADUATION                           : np.full((num_years), 0).tolist(),        
        hwmvf.HW_STAFF_TARGET_RETENT_INCENT_LEVELS                 : [],
    }

def HW_init_staff_type_records(facility_type_records, num_years, costing_mode):
    staff_type_records = []

    num_facility_types = len(facility_type_records)

    # Health service providers
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_GENERAL_MEDICAL_PRACTITIONERS_MST_ID), 
            'General medical practitioners', 
            'GB_stGenMedPractitioners', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_GENERAL_MEDICAL_PRACTITIONERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_SPECIAL_MEDICAL_PRACTITIONERS_MST_ID), 
            'Specialist medical practitioners', 
            'GB_stSpecialistMedPractitioners', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_SPECIAL_MEDICAL_PRACTITIONERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    # Rachel: No longer exist in ISCO list.
    # staff_type_dict_list.append(
    #     HR_create_staff_type_rec(
    #         str(hrc.HR_OB_GYNS_MST_ID), 
    #         'Ob\Gyns', 
    #         'GB_stObGyns',
    #         True, 
    #         hrc.HR_HLTH_SERV_PROVIDER_MST_ID,
    #         hrc.HR_OB_GYNS_MST_ID, 
    #         num_facility_types, 
    #         num_years 
    #     )
    # )
    # staff_type_dict_list.append(
    #     HR_create_staff_type_rec(
    #         str(hrc.HR_PEDIATRICIANS_MST_ID), 
    #         'Pediatricians', 
    #         'GB_stPediatricians', 
    #         True,
    #         hrc.HR_HLTH_SERV_PROVIDER_MST_ID,
    #         hrc.HR_PEDIATRICIANS_MST_ID, 
    #         num_facility_types, 
    #         num_years 
    #     )
    # )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_NURSING_PROFESSIONALS_MST_ID), 
            'Nursing professionals', 
            'GB_stNursingProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_NURSING_PROFESSIONALS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MIDWIFERY_PROFESSIONALS_MST_ID), 
            'Midwifery professionals', 
            'GB_stMidwiferyProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_MIDWIFERY_PROFESSIONALS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_NURSING_ASSOCIATE_PROFESSIONALS_MST_ID), 
            'Nursing associate professionals', 
            'GB_stNursingAssocProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_NURSING_ASSOCIATE_PROFESSIONALS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_MST_ID), 
            'Midwifery associate professionals', 
            'GB_stMidwiferyAssocProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_DENTISTS_MST_ID), 
            'Dentists', 
            'GB_stDentists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_DENTISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_DENTAL_ASSIST_THERAPISTS_MST_ID), 
            'Dental assistants and therapists', 
            'GB_stDentalAssistTherapists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_DENTAL_ASSIST_THERAPISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MED_DENTAL_PROSTHETIC_REL_TECHS_MST_ID), 
            'Medical and dental prosthetic and related technicians', 
            'GB_stMedDentalPostheticRelTechs', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_MED_DENTAL_PROSTHETIC_REL_TECHS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PHARMACISTS_MST_ID), 
            'Pharmacists', 
            'GB_stPharmacists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PHARMACISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PHARM_TECHS_ASSIST_MST_ID), 
            'Pharmaceutical technicians and assistants', 
            'GB_stPharmTechsAssistants', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PHARM_TECHS_ASSIST_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        ))
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PARAMED_PRACTITIONERS_MST_ID), 
            'Paramedical practitioners', 
            'GB_stParamedPractitioners', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PARAMED_PRACTITIONERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        ))
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MED_IMAGING_THERA_EQUIP_TECHS_MST_ID), 
            'Medical imaging and therapeutic equipment technicians', 
            'GB_stMedImagingTherapeuticEquipTechs', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_MED_IMAGING_THERA_EQUIP_TECHS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MED_PATH_LAB_TECHS_MST_ID), 
            'Medical and pathology laboratory technicians', 
            'GB_stMedPathLabTechs', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_MED_PATH_LAB_TECHS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_MST_ID), 
            'Environmental and occupational health and hygiene professionals', 
            'GB_stEnvironOccupatHealthHygieneProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_DIETICIANS_NUTRITIONISTS_MST_ID), 
            'Dieticians and nutritionists', 
            'GB_stDieticiansNutritionists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_DIETICIANS_NUTRITIONISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_AUDIOLOGISTS_SPEECH_THERAPISTS_MST_ID), 
            'Audiologists and speech therapists', 
            'GB_stAudiologistsSpeechTherapists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_AUDIOLOGISTS_SPEECH_THERAPISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_MST_ID), 
            'Optometrists and ophthalmic opticians', 
            'GB_stOptometristsOpthalmicOpticians', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_DISPENSING_OPTICIANS_MST_ID), 
            'Dispensing opticians', 
            'GB_stDispensingOpticians', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_DISPENSING_OPTICIANS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_HLTH_PROF_NOT_ELSEWHERE_CLASSIFIED_MST_ID), 
            'Health professionals not elsewhere classified', 
            'GB_stHealthProfNotElsewhereClassified', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_HLTH_PROF_NOT_ELSEWHERE_CLASSIFIED_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_MST_ID), 
            'Environmental and occupational health inspectors and associates', 
            'GB_stEnvironOccupatHealthInspectorsAssoc', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_TRAD_COMP_MED_PROF_MST_ID), 
            'Traditional and complementary medicine professionals', 
            'GB_stTradComplementaryMedicineProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_TRAD_COMP_MED_PROF_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_TRAD_COMP_MED_ASSOC_PROF_MST_ID), 
            'Traditional and complementary medicine associate professionals', 
            'GB_stTraditionalCompMedAssocProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_TRAD_COMP_MED_ASSOC_PROF_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_COMM_HLTH_WORKERS_MST_ID), 
            'Community health workers', 
            'GB_stCommHealthWorkersLowercase', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_COMM_HLTH_WORKERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_HEALTH_CARE_ASSISTS_MST_ID), 
            'Health care assistants', 
            'GB_stHealthCareAssists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_HEALTH_CARE_ASSISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_HOME_BASED_PERS_CARE_WORKERS_MST_ID), 
            'Home-based personal care workers', 
            'GB_stHomeBasedPersCareWorkers', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_HOME_BASED_PERS_CARE_WORKERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_MST_ID), 
            'Personal care workers in health services not elsewhere classified', 
            'GB_stPersCareWorkersHealthServNotElseClassified', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PHYSIOTHERAPISTS_MST_ID), 
            'Physiotherapists', 
            'GB_stPhysiotherapists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PHYSIOTHERAPISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PHYSIOTHERAPY_TECHS_ASSISTS_MST_ID), 
            'Physiotherapy technicians and assistants', 
            'GB_stPhysiotherapyTechsAssists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PHYSIOTHERAPY_TECHS_ASSISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MED_REC_HEALTH_INFO_TECHS_MST_ID), 
            'Medical records and health information technicians', 
            'GB_stMedRecHealthInfoTechs', 
            True,
            hwc.HW_HLTH_MGMT_SUPP_PERS_MST_ID,
            hwc.HW_MED_REC_HEALTH_INFO_TECHS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_CLINIC_OFF_SURG_TECHS_MST_ID), 
            'Medical assistants', 
            'GB_stMedAssist', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_CLINIC_OFF_SURG_TECHS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_AMBULANCE_WORKERS_MST_ID), 
            'Ambulance workers', 
            'GB_stAmbulanceWorkers', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_AMBULANCE_WORKERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_HEALTH_ASSOC_PROF_NOT_CLASS_MST_ID), 
            'Health associate professionals not elsewhere classified', 
            'GB_stHealthAssocProfNotElseClassified', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_HEALTH_ASSOC_PROF_NOT_CLASS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_PSYCHOLOGISTS_MST_ID), 
            'Psychologists', 
            'GB_stPsychologists', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_PSYCHOLOGISTS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode 
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_SOCIAL_WORK_COUNSELLING_PROF_MST_ID), 
            'Social work and counselling professionals', 
            'GB_stSpecialWorkCounsProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_SOCIAL_WORK_COUNSELLING_PROF_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_SOCIAL_WORK_ASSOC_PROF_MST_ID), 
            'Social work associate professionals', 
            'GB_stSocialWorkAssocProf', 
            True,
            hwc.HW_HLTH_SERV_PROVIDER_MST_ID,
            hwc.HW_SOCIAL_WORK_ASSOC_PROF_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_BIOMED_ENGINEER_MST_ID), 
            'Biomedical engineer', 
            'GB_stBiomedEngineer', 
            True,
            hwc.HW_HLTH_MGMT_SUPP_PERS_MST_ID,
            hwc.HW_BIOMED_ENGINEER_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_MED_SECRETARIES_MST_ID), 
            'Medical secretaries', 
            'GB_stMedicalSecretaries', 
            True,
            hwc.HW_HLTH_MGMT_SUPP_PERS_MST_ID,
            hwc.HW_MED_SECRETARIES_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_HLTH_SERV_MANAGERS_MST_ID), 
            'Health service managers', 
            'GB_stHealthServManagers', 
            True,
            hwc.HW_HLTH_MGMT_SUPP_PERS_MST_ID,
            hwc.HW_HLTH_SERV_MANAGERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )
    staff_type_records.append(
        HW_create_staff_type_rec(
            str(hwc.HW_AGED_CARE_SERVICE_MANAGERS_MST_ID), 
            'Aged care service managers', 
            'GB_stAgedCareServManagers', 
            True,
            hwc.HW_HLTH_MGMT_SUPP_PERS_MST_ID,
            hwc.HW_AGED_CARE_SERVICE_MANAGERS_MST_ID, 
            num_facility_types, 
            num_years,
            costing_mode
        )
    )

    # Defunct
    # staff_type_dict_list.append(
    #     create_staff_type_dict(
    #         str(hrc.HR_NON_HLTH_PROFESSIONALS_MST_ID), 
    #         'Non-health professionals not elsewhere classified', 
    #         'GB_stNonHealthProfessionals', 
    #         True,
    #         hrc.HR_HLTH_MGMT_SUPP_PERS_MST_ID,
    #         hrc.HR_NON_HLTH_PROFESSIONALS_MST_ID, 
            # num_facility_types, 
            # num_years 
    #     )
    # )
    # staff_type_dict_list.append(
    #     create_staff_type_dict(
    #         str(hrc.HR_NON_HLTH_TECHS_ASSOC_PROF_MST_ID), 
    #         'Non-health technicians and associate professionals not elsewhere classified', 
    #         'GB_stNonHealthTechsAssocProf', 
    #         True,
    #         hrc.HR_HLTH_MGMT_SUPP_PERS_MST_ID,
    #         hrc.HR_NON_HLTH_TECHS_ASSOC_PROF_MST_ID, 
            # num_facility_types, 
            # num_years 
    #     )
    # )
    # staff_type_dict_list.append(
    #     create_staff_type_dict(
    #         str(hrc.HR_CLERICAL_SUPP_WORKERS_MST_ID), 
    #         'Clerical support workers', 
    #         'GB_stClericalSuppWorkers', 
    #         True,
    #         hrc.HR_HLTH_MGMT_SUPP_PERS_MST_ID,
    #         hrc.HR_CLERICAL_SUPP_WORKERS_MST_ID, 
            # num_facility_types, 
            # num_years 
    #     )
    # )
    # staff_type_dict_list.append(
    #     create_staff_type_dict(
    #         str(hrc.HR_PLANT_MACH_OPER_ASSEMB_MST_ID), 
    #         'Plant and machine operators and assemblers', 
    #         'GB_stPlantMachOperAssemb', 
    #         True,
    #         hrc.HR_HLTH_MGMT_SUPP_PERS_MST_ID,
    #         hrc.HR_PLANT_MACH_OPER_ASSEMB_MST_ID, 
            # num_facility_types, 
            # num_years 
    #     )
    # )
    # staff_type_dict_list.append(
    #     create_staff_type_dict(
    #         str(hrc.HR_ELEMENTARY_OCCUPATIONS_MST_ID), 
    #         'Elementary occupations', 
    #         'GB_stElementaryOccupations', 
    #         True,
    #         hrc.HR_HLTH_MGMT_SUPP_PERS_MST_ID,
    #         hrc.HR_ELEMENTARY_OCCUPATIONS_MST_ID, 
            # num_facility_types, 
            # num_years 
    #     )
    # )
    
    return staff_type_records

def HW_init_staff_type_target_setting_methods(staff_type_records):
    staff_type_rec = hwmvu.get_HW_staff_type_rec_from_curr_ID(staff_type_records, hwc.HW_GENERAL_MEDICAL_PRACTITIONERS_CURR_ID)
    hwmvu.set_HW_staff_type_target_setting_method(staff_type_rec, hwc.HW_STAFF_LEVEL_FACILITY_CURR_ID, hwc.HW_EXISTING_PLANS_MST_ID)
    
    staff_type_rec = hwmvu.get_HW_staff_type_rec_from_curr_ID(staff_type_records, hwc.HW_PHARM_TECHS_ASSIST_CURR_ID)
    hwmvu.set_HW_staff_type_target_setting_method(staff_type_rec, hwc.HW_STAFF_LEVEL_NATIONAL_CURR_ID, hwc.HW_EXISTING_PLANS_MST_ID)
    
    staff_type_rec = hwmvu.get_HW_staff_type_rec_from_curr_ID(staff_type_records, hwc.HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_CURR_ID)
    hwmvu.set_HW_staff_type_target_setting_method(staff_type_rec, hwc.HW_STAFF_LEVEL_REGIONAL_CURR_ID, hwc.HW_EXISTING_PLANS_MST_ID)
    
    # staff_type_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_HLTH_SERV_MANAGERS_CURR_ID)
    # hrmvu.set_HR_staff_type_target_setting_method(staff_type_rec, hrc.HR_STAFF_LEVEL_NATIONAL_CURR_ID, hrc.HR_EXISTING_PLANS_MST_ID)
    
    # staff_type_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_HEALTH_MGMT_PROFESSIONALS_CURR_ID)
    # hrmvu.set_HR_staff_type_target_setting_method(staff_type_rec, hrc.HR_STAFF_LEVEL_REGIONAL_CURR_ID, hrc.HR_EXISTING_PLANS_MST_ID)
    
    # staff_type_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_CLERICAL_SUPP_WORKERS_CURR_ID)
    # hrmvu.set_HR_staff_type_target_setting_method(staff_type_rec, hrc.HR_STAFF_LEVEL_REGIONAL_CURR_ID, hrc.HR_EXISTING_PLANS_MST_ID)
    
    # staff_type_rec = hrmvu.get_HR_staff_type_rec_from_curr_ID(staff_type_records, hrc.HR_PLANT_MACH_OPER_ASSEMB_CURR_ID)
    # hrmvu.set_HR_staff_type_target_setting_method(staff_type_rec, hrc.HR_STAFF_LEVEL_NATIONAL_CURR_ID, hrc.HR_EXISTING_PLANS_MST_ID)

#######################################################################################################
#                                                                                                     #
#                                       In-service training types                                     #
#                                                                                                     #
#######################################################################################################

def HW_create_inserv_training_type_rec(num_years = int):

    return {
        hwmvf.HW_INSERV_TRAIN_STAFF_TYPE_TO_TRAIN    : str(hwc.HW_GENERAL_MEDICAL_PRACTITIONERS_MST_ID),
        hwmvf.HW_INSERV_TRAIN_NAME                   : '',
        hwmvf.HW_INSERV_TRAIN_COST_PER_STAFF_TRAINED : 0.0,
        hwmvf.HW_INSERV_TRAIN_NUM_STAFF_TO_TRAIN     : np.full((num_years), 0).tolist(),   
    }

def HW_init_inserv_train_type_records(num_years):
    inserv_train_type_dict_list = []

    for tt in acu.GBRange(1, hwc.HW_NUM_INSERV_TRAIN_TYPES):
        inserv_train_type_dict = HW_create_inserv_training_type_rec(num_years)
        inserv_train_type_dict_list.append(inserv_train_type_dict)

    return inserv_train_type_dict_list

#######################################################################################################
#                                                                                                     #
#                                       Attrition types                                               #
#                                                                                                     #
#######################################################################################################

def HW_create_attrition_type_rec(ID = str, name = str):

    return {
        hwmvf.HW_ATTRIT_TYPE_ID   : ID,
        hwmvf.HW_ATTRIT_TYPE_NAME : name,
    }

#######################################################################################################
#                                                                                                     #
#                                       Retention incentives                                          #
#                                                                                                     #
#######################################################################################################

def HW_create_retent_incent_rec(ID = str, name = str, num_attrit_types = int):

    retent_incent_dict =  {
        hwmvf.HW_RETENT_INCENT_ID                : ID,
        hwmvf.HW_RETENT_INCENT_NAME              : name,
        hwmvf.HW_RETENT_INCENT_PERC_REDUC_ATTRIT : [],
    }

    if num_attrit_types > 0:
        retent_incent_dict[hwmvf.HW_RETENT_INCENT_PERC_REDUC_ATTRIT] = np.full((num_attrit_types), 0).tolist()

    return retent_incent_dict

########################################################################################################################################
########################################################################################################################################
#                                                                                                                                      #
#                                                       Meta data                                                                      #
#                                                                                                                                      #
########################################################################################################################################
########################################################################################################################################

def HW_init_static_staff_list_meta():
    static_staff_list_meta = []

    for st in acu.GBRange(1, hwc.HW_NUM_STATIC_STAFF_TYPES):
        mst_ID = hwu.get_HW_static_staff_mst_ID(st)

        static_staff_list_meta.append({
            "mstID"          : mst_ID,
            "name"           : hwu.get_HW_static_staff_name(mst_ID),
            "stringConstant" : hwu.get_HW_static_staff_string_constant(mst_ID)
        })
    
    return static_staff_list_meta

#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################

def HW_init_staff_level_staff_type_year_arr(num_staff_types = int, num_years = int, value = 0.0):
    return np.full((hwc.HW_NUM_STAFFING_LEVELS, num_staff_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def HW_init_staff_level_staff_type_compens_year_arr(num_staff_types = int, num_years = int, value = 0.0):
    return np.full((hwc.HW_NUM_STAFFING_LEVELS, num_staff_types, hwc.HW_NUM_COMPENS_TYPES, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def HW_init_staff_type_year_arr(num_staff_types = int, num_years = int, value = 0.0):
    return np.full((num_staff_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def HW_init_inserv_train_type_year_arr(num_years = int, value = 0.0):
    return np.full((hwc.HW_NUM_INSERV_TRAIN_TYPES, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


from SpectrumCommon.Const.IV.IVConst import *
from SpectrumCommon.Const.IV.IVDatabaseKeys import *

from SpectrumCommon.Const.IH.IHConst import *
from SpectrumCommon.Const.IH.IHTags import *

from SpectrumCommon.Const.CS.CSTags import *

from SpectrumCommon.Const.FP.FPConst import *
from SpectrumCommon.Const.FP.FPTags import *

from SpectrumCommon.Const.DP.DPTags import *

##############################    DP    ##########################

RP_DP_BIG_POP_TAG = DP_BigPopTag
RP_DP_TFR_TAG = DP_TFRTag

##############################    FP    ##########################

RP_FP_ACTIVE_METHODS_TAG = FP_ActiveMethodsTag
RP_FP_EFFECTIVENESS_TAG  = FP_EffectivenessTag
RP_FP_METHOD_MIX_TAG     = FP_MethodMixTag
RP_FP_USERS_TAG          = FP_UsersTag
RP_FP_PREVALENCE_TAG     = FP_PrevalenceTag
RP_FP_BIRTHS_TAG         = FP_BirthsTag

RP_FP_NUM_METHODS = FP_Max_Methods

RP_FP_ALL_AGES = FP_All_Ages
RP_FP_ALL_NEED = FP_all_need

# These are all possible active FamPlan methods that have default values for FamPlan (substantially shorter than the full list of FamPlan methods)

RP_FP_M_CONDOM_M_CURR_ID             = FP_CondomML
RP_FP_M_STERILIZ_F_CURR_ID           = FP_fster          
RP_FP_M_STERILIZ_M_CURR_ID           = FP_mster       
RP_FP_M_IUD_COPPER_10_YR_CURR_ID     = FP_IUDCopper     
RP_FP_M_INJECT_DEPO_3_MO_CURR_ID     = FP_inj3month 
RP_FP_M_IMPLANT_JADELLE_5_YR_CURR_ID = FP_Jadelle
RP_FP_M_PILL_STANDARD_CURR_ID        = FP_StandardPill
RP_FP_M_VAG_BARRIER_CURR_ID          = FP_vag_barrier
RP_FP_M_OTHER_MODERN_METHODS_CURR_ID = FP_other

# Traditional methods that are not modelled by RAPID
RP_FP_M_TRAD_WITHDRAWAL_CURR_ID    = FP_withdrawal   
RP_FP_M_TRAD_PERIODIC_ABST_CURR_ID = FP_PerAbst        
RP_FP_M_TRAD_OTHER_CURR_ID         = FP_Traditional
RP_FP_M_TRAD_METHODS               = FP_TRAD_METH


def InterventionScenario(p,r,IntvVec): #function [p1,r1]=InterventionScenario(p,r,IntvVec)

    #%r_cs:           Rate-of-presentation to care, first careseeking visit for symptomatic TB
    #%r_cs2:          Rate-of-presentation to care, second and subsequent careseeking visits for symptomatic TB
    #%r.cs3:          Per-capita rate of casefinding amongst sub-clinical TB
    #%p_pu:           Proportion of careseeking visits that are to the public sector
    #%p_Dx            Probability of successful TB diagnosis and treatment initiation per careseeking visit (in public and private sector)
    #%r_progression:	 Per-capita annual rate of progression from �fast� latent infection (differentiated by HIV-status, h0, h1, hart)
    #%r_reactivation: Per-capita annual rate of reactivation from �slow� latent infection  (differentiated by HIV-status, h0, h1, hart)
    #%r_Tx2:          Per-capita annual rate of second-tline treatment completion
    #%p_cure2:        Proportion cure after successful completion of SL treatment 
    #%p_TX_complete:  Proportion treatment completion (used to estimate per-capita annual rate of treatment interruption (r.default) in public and private sector)
    #%p_MDRrec2019:	 Of diagnosed TB with rifampcin resistance, proportion that is recognised as such in 2019 (throgh DST)
    #%r_relapse(1):	 Per-capita annual rate of relapse in first two years after treatment completion
    #%r_relapse(2):	 Per-capita annual rate of relapse in first two years after self-cure or incomplete treatment
    #%r_relapse(3)	 Per-capita annual rate of relapse >two years after last TB episode
    #%p_VE(1):        Vaccine efficacy on reduction of susceptibility (pre-exposure protection)
    #%p_VE(2):        Vaccine efficacy on reduction of progression to active disease (post-exposure protection)
    #%r_vacc:         Per-capita annual rate of vaccination
    #%r_waning:       Per-capita annual rate of waning vaccine immunity

    #%copy model parameters
    p1=p.copy() #p1=p;
    r1=r.copy() #r1=r;
    fn = list(IntvVec) #fn = fieldnames(IntvVec);
    for k in fn :#for k=1:numel(fn)
        if len(IntvVec[k])!=0  : #if( ~isempty(IntvVec.(fn{k})) )
            if k == 'r_cs':#if( strcmp(fn{k},'r_cs') ),
              factor=IntvVec[k] #factor=IntvVec.(fn{k});
              if(sum(factor)==0): #if(sum(factor)==0),
                  r1['pu']=0 #r1.pu=0;
                  #%r1.ART_init = 0; 
                  #%p1.MDR_rec = [0 0]; 
              #end
              for J in range(len(factor)): #for J=1:length(factor)  
                if(factor[J]>1): #if(factor(J)>1)  
                    r1['cs'][J]=r1['cs'][J]*factor[J] #r1.cs(J)=r1.cs(J)*factor(J);
                elif (factor[J]<1): #elseif (factor(J)<1)
                    r1['cs'][J]  = r1['cs_1997'][J] + factor[J]*(r1['cs'][J]-r1['cs_1997'][J]) # r1.cs(J)  = r1.cs_1997(J) + factor(J)*(r1.cs(J)-r1.cs_1997(J));
                    #%r1.pu     = r1.pu*factor(1);
                #end 
              #end
              #r1['cs']=r1['cs']*IntvVec[k]
            #end
          
            if k=='r_cs2':#if( strcmp(fn{k},'r_cs2') )#if( strcmp(fn{k},'r_cs2') ),
              factor = IntvVec[k] #factor=IntvVec.(fn{k});
              if(sum(factor)==0): #if(sum(factor)==0),
                  r1['pu']=0 #r1.pu=0;
                  #%r1.ART_init = 0; 
                  #%p1.MDR_rec = [0 0];
              #end
              for J in range(len(factor)): #for J=1:length(factor)
                if(factor[J]>1): #if(factor(J)>1)  
                    r1['cs2'][J] = r1['cs2'][J]*factor[J] #r1.cs2(J)=r1.cs2(J)*factor(J);
                elif (factor[J]<1): #elseif (factor(J)<1)
                    r1['cs2'][J]  = r1['cs2_1997'][J] + factor[J]*(r1['cs2'][J]-r1['cs2_1997'][J]) #r1.cs2(J)  = r1.cs2_1997(J) + factor(J)*(r1.cs2(J)-r1.cs2_1997(J)); 
                #end
              #end
              #r1['cs2'] = r1['cs2']*IntvVec[k]#r1.cs2=r1.cs2.*IntvVec.(fn{k});
            #end
         
            if k=='r_cs3':#if( strcmp(fn{k},'r_cs3') ),
              r1['cs3']=r1['cs3']*IntvVec[k]#r1.cs3=r1.cs3.*IntvVec.(fn{k});
            #end
          
            if k=='p_pu': #if( strcmp(fn{k},'p_pu') ),
              p1['pu']=IntvVec[k] #p1.pu=IntvVec.(fn{k});
            #end
          
            if k=='p_Dx': #if( strcmp(fn{k},'p_Dx') ),
              p1['Dx']=IntvVec[k]#p1.Dx=IntvVec.(fn{k});
            #end

            if k=='p_PT_PLHIV': #if( strcmp(fn{k},'p_Dx') ),
              p1['PT_PLHIV']=(1-IntvVec[k])
            #end

            if k=='r_progression':#if( strcmp(fn{k},'r_progression') ),
              r1['progression']=r1['progression']*(1-IntvVec[k])#r1.progression=r1.progression.*(1-IntvVec.(fn{k}));
            #end
          
            if k=='r_reactivation': #if( strcmp(fn{k},'r_reactivation') ),
              r1['reactivation']=r1['reactivation']*(1-IntvVec[k])#r1.reactivation=r1.reactivation.*(1-IntvVec.(fn{k}));
            #end
          
            if k=='r_Tx2' :#if( strcmp(fn{k},'r_Tx2') ),
              r1['Tx2']=IntvVec[k]#r1.Tx2=IntvVec.(fn{k});
            #end
          
            if k=='p_cure': #if( strcmp(fn{k},'p_cure') ),
              #%p1.cure=p1.cure*(1-IntvVec.(fn{k}));
              for i,e in enumerate(p1['cure']):
                p1['cure'][i] = 1-(1-p1['cure'][i])*(1-IntvVec[k]) #p1.cure=1-(1-p1.cure)*(1-IntvVec.(fn{k}));
            #end
          
            if k=='p_cure2' :#if( strcmp(fn{k},'p_cure2') ),
              for i,e in enumerate(p1['cure2']):
                p1['cure2'][i]=1-(1-p1['cure2'][i])*(1-IntvVec[k])#p1.cure2=1-(1-p1.cure2)*(1-IntvVec.(fn{k}));
            #end
          
            if k=='r_default': #if( strcmp(fn{k},'r_default') ),
              r1['default'][0] = r1['default'][0]*(1-IntvVec[k])#r1.default(1)=r1.default(1)*(1-IntvVec.(fn{k}));
            #end
          
            if k=='r_default2' :#if( strcmp(fn{k},'r_default2') ),
              r1['default2'][0] = r1['default2'][0]*(1-IntvVec[k]) #r1.default2(1)=r1.default2(1)*(1-IntvVec.(fn{k}));
            #end
         
            if k=='p_TX_complete': #if( strcmp(fn{k},'p_TX_complete') ),
              p1['TX_complete'] = IntvVec[k] #p1.TX_complete=IntvVec.(fn{k});
            #end
          
            if k=='p_MDRrec':#if( strcmp(fn{k},'p_MDRrec') ),
              p1['MDR_rec']=IntvVec[k] #p1.MDR_rec=IntvVec.(fn{k});
            #end
          
            if k=='r_relapse_1': #if( strcmp(fn{k},'r_relapse_1') ),
              r1['relapse'][0]=IntvVec[k]#r1.relapse(1)=IntvVec.(fn{k});
            #end
          
            if k=='r_relapse_2': #if( strcmp(fn{k},'r_relapse_2') ),
              r1['relapse'][1]=IntvVec[k] #r1.relapse(2)=IntvVec.(fn{k});
            #end
   
            if k=='r_relapse_3': #if( strcmp(fn{k},'r_relapse_3') ),
               r1['relapse'][2]=IntvVec[k]#r1.relapse(3)=IntvVec.(fn{k});
            #end
          
            if k=='p_VE_1':#if( strcmp(fn{k},'p_VE_1') ),
              p1['VE'][0]=IntvVec[k] #p1.VE(1)=IntvVec.(fn{k});
            #end
          
            if k=='p_VE_2':#if( strcmp(fn{k},'p_VE_2') ),
              p1['VE'][1]=IntvVec[k]#p1.VE(2)=IntvVec.(fn{k});
            #end
          
            if k=='r_vacc': #if( strcmp(fn{k},'r_vacc') ),
              r1['vacc']=IntvVec[k] #r1.vacc=IntvVec.(fn{k});
            #end
          
            if k=='r_waning': #if( strcmp(fn{k},'r_waning') ),
              r1['waning']=IntvVec[k] #r1.waning=IntvVec.(fn{k});
            #end
          
            if k=='r_ART_init': #if( strcmp(fn{k},'r_ART_init') ),
              r1['ART_init'] = r1['ART_init']*IntvVec[k] #r1.ART_init=r1.ART_init*IntvVec.(fn{k});
            #end
        #end
    #end
    return([p1,r1])


from SpectrumCommon.Const.IS import *
import SpectrumCommon.Const.IS.ISTags as ismvt
from SpectrumCommon.Modvars.GB.GBDefs import *

is_facility_dim = 'is_facility'
is_facility_ids = {}

is_pc_standard_rows_dim = 'is_pc_standard_rows_dim'
is_pc_standard_rows_id = {}

is_rehab_dim='is_rehab'
is_rehab_ids={
        
        IS_REHAB_SMALL_SCALE_CURR_ID         : 'Small',
        IS_REHAB_MEDIUM_SCALE_CURR_ID        : 'Medium',
        IS_REHAB_LARGE_SCALE_CURR_ID         : 'Large',
        IS_REHAB_MAINT_REPLACE_EQUIP_CURR_ID : 'Maintenance and equipment replacement',
}
is_facility_equip_types_dim = 'is_facility_equip_types'
is_facility_equip_types_ids = {
        IS_FACIL_MEDICAL_EQUIP_CURR_ID   : 'Medical',
        IS_FACIL_FURNITURE_EQUIP_CURR_ID : 'Furniture',
}
is_program_areas_dim = 'is_program_areas'
is_program_areas_indices = {}

is_med_equip_dim = 'is_med_equip'
is_med_equip_ids = {}

is_med_equip_group_dim = 'is_med_equip_group'
is_med_equip_group_ids = {}

is_furniture_dim = 'is_furniture'
is_furniture_ids = {}
is_vehicle_dim = 'is_vehicle'
is_vehicle_ids = {}
is_ict_dim = 'is_ict'
is_ict_ids = {}

IS_modvar_shapes = {
    
        #       
        ismvt.IS_VERSION_TAG_V1                               : gb_int_shape('IS Version'),
        ismvt.IS_DATA_CHANGED_TAG_V1                          : gb_bool_shape('IS Data Changed'), 
        ismvt.IS_PROJ_VALID_TAG_V1                            : gb_bool_shape('IS Project Valid'), 
        ismvt.IS_FILE_NAME_TAG_V1                             : gb_str_shape('File name'),
        ismvt.IS_PROJ_VALID_DATE_TAG_V1                       : gb_str_shape('IS projection valid date'),
        ismvt.IS_FACILITY_TYPE_RECORDS_TAG                    : gb_shape('IS Facility Type Records', '1D dict',
                                                                         dim=[is_facility_dim],
                                                                         indicies=[is_facility_ids],
                                                                         innerShape={
                                                                             'facilityTypeID' :gb_str_shape('facility type id'), 
                                                                             'stringConstant': gb_str_shape('string constant'), 
                                                                             'name': gb_str_shape('Name'), 
                                                                             'active':gb_bool_shape('Is active flag'), 
                                                                             'custom': gb_bool_shape('Is custom facility flag'), 
                                                                             'supportTypeMstID':gb_int_shape('support type master ID'), 
                                                                             'delivChanMstID':gb_str_shape('Delivery channel master id'), 
                                                                             'defFacilityTypeMapMstID':gb_int_shape('default facility type map master ID'), 
                                                                             'totalNumBaseline':gb_int_shape('Total number baseline'), 
                                                                             'avgNumBeds':gb_float_shape('Average number of beds'), 
                                                                             'avgOccupancyRateBeds':gb_float_shape('Average occupancy rate'), 
                                                                             'avgNumOutpatVisitsPerYear':gb_float_shape('Average outpatient visits per year'), 
                                                                             'yearsToBuild':gb_int_shape('Year to build'), 
                                                                             'totalConstructCost':gb_float_shape('Total construction cost'), 
                                                                             'landCost':gb_float_shape('Land cost'), 
                                                                             'constructionCostPerSqM':gb_float_shape('Construction cost per square meter'), 
                                                                             'avgSizeSqM':gb_float_shape('Average size square meter'), 
                                                                             'rehabCost':gb_shape('Rehab cost', '1D float', dim=[is_rehab_dim], indicies=[is_rehab_ids]), 
                                                                             'totalOpCost':gb_float_shape('Total operating cost'), 
                                                                             'totalOpCostAsPercOfConstructCost':gb_float_shape('total operating cost as percent of construction cost'), 
                                                                             'desiredPopPerFacilityRatioPopNorms':gb_float_shape('desired population per facility ratio'), 
                                                                             'targScaleUpPopNorms' : gb_year_shape('Target scale up population norms'), 
                                                                             'targScaleUpExistingPlansExpectedOp':gb_year_shape('Taget scale up existing plans expected Op'), 
                                                                             'targScaleUpExistingPlansBeginConstruct':gb_year_shape('taget scaleup existing plans begin construct'), 
                                                                             'targNumToRehab':gb_shape('Target number to rehab', '1D float', dim=[is_rehab_dim], indicies=[is_rehab_ids]), 
                                                                             'targNumToRehabByYear':gb_shape('Rehab cost', '2D float', dim=[is_rehab_dim, gb_year_dim], indicies=[is_rehab_ids, gb_year_indices]), 
                                                                             'numToEquip':gb_shape('Number to equip', '1D float',
                                                                                                   dim=[gb_year_dim], 
                                                                                                   indicies=[gb_year_indices]), #Todo: if iht is in specific plan mode then this key/value is 2d.  
                                                                             'equipmentPurchaseCost':gb_shape('Target number to rehab', '1D float', dim=[is_facility_equip_types_dim], indicies=[is_facility_equip_types_ids]), 
                                                                         }
                                                                ),
        ismvt.IS_FACILITY_MED_EQUIP_RECORDS_TAG               : gb_shape('IS Medical equipment Records', '1D dict',
                                                                         dim=[is_med_equip_dim],
                                                                         indicies=[is_med_equip_ids],
                                                                         innerShape={
                                                                                'facilityEquipID' : gb_str_shape('Facility equipment ID'),
                                                                                'custom' : gb_bool_shape('Is custom equipment flag'),
                                                                                'name' : gb_str_shape('Name'),
                                                                                'stringConstant' : gb_str_shape('String constant'),
                                                                                'facilityTypeID' : gb_str_shape('Facility type ID'),   
                                                                                'unitCost' : gb_float_shape('Unit cost'),
                                                                                'numUnits' : gb_int_shape('Number of units'),
                                                                                'workingLife' : gb_float_shape('Working life'),
                                                                                'facilityMedEquipGroupID' : gb_str_shape('Facility medical equipment group ID'),
                                                                 
                                                                        }
                                                                ),       
        ismvt.IS_FACILITY_MED_EQUIP_GROUP_RECORDS_TAG         : gb_shape('IS Medical equipment group Records', '1D dict',
                                                                         dim=[is_med_equip_group_dim],
                                                                         indicies=[is_med_equip_group_ids],
                                                                         innerShape={
                                                                             'facilityMedEquipGroupID' : gb_str_shape('Facility medical equipment group ID'),
                                                                             'name' : gb_str_shape('Name'),
                                                                             'stringConstant' : gb_str_shape('String constant'),
                                                                             'facilityTypeID' : gb_str_shape('Facility type ID'),                                                                      
                                                                        }
                                                                ),  
        ismvt.IS_FACILITY_FURNITURE_RECORDS_TAG_V1            : gb_shape('IS furniture Type Records', '1D dict',
                                                                         dim=[is_furniture_dim],
                                                                         indicies=[is_furniture_ids],
                                                                         innerShape={
                                                                             'facilityEquipID' : gb_str_shape('Facility equipment ID'),
                                                                             'custom' : gb_bool_shape('Is custom furniture flag'),
                                                                             'name' : gb_str_shape('Name'),
                                                                             'stringConstant' : gb_str_shape('String constant'),
                                                                             'facilityTypeID' : gb_str_shape('Facility type ID'),
                                                                             'unitCost' : gb_float_shape('Unit cost'),
                                                                             'numUnits' : gb_int_shape('Number of units'),
                                                                             'workingLife' : gb_float_shape('Working life')
                                                                        }
                                                                ),       
        ismvt.IS_ENTER_CONSTRUCT_COSTS_BY_COMP_TAG_V1         : gb_bool_shape('Enter construct costs by component'),  
        ismvt.IS_ENTER_OP_COST_PERC_CONSTRUCT_COST_TAG_V1     : gb_bool_shape('Enter operating costs as percentage of construct costs'),        
        ismvt.IS_FACIL_TARGET_SETTING_METHOD_TAG_V1           : gb_int_shape('Facility target setting method'),  
        ismvt.IS_VEHICLE_TYPE_RECORDS_TAG_V1                  : gb_shape('IS Vehicle Type Records', '1D dict',
                                                                         dim=[is_vehicle_dim],  
                                                                         indicies=[is_vehicle_ids],
                                                                         innerShape={
                                                                                        'vehicleTypeID' : gb_str_shape('Vehicle type ID'),
                                                                                        'name' : gb_str_shape('Name'),
                                                                                        'currNumber' : gb_float_shape('Current number'),
                                                                                        'unitCost' : gb_float_shape('Unit cost'),
                                                                                        'workingLifeInYears' : gb_float_shape('Working life in years'),
                                                                                        'driverSalary' : gb_float_shape('Driver salary'),
                                                                                        'fuelConsumption' : gb_float_shape('Fuel consumption'),
                                                                                        'fuelCostDirectEntry' : gb_float_shape('Fuel cost direct entry'),
                                                                                        'distanceDrivenPerYear' : gb_float_shape('Distance driven per year'),
                                                                                        'repairAndMaintPercCapCost' : gb_float_shape('Repair and maintenance percent capital cost'),
                                                                                        'targNumPerFacility' : gb_shape('Target number per facility', '1D float', dim=[is_facility_dim], indicies=[is_facility_ids]),
                                                                                        'targNumPerFacilityOtherNonFacBased' : gb_int_shape('Target number per facility other non-facility based'),
                                                                                }
                                                                ),
        ismvt.IS_DIRECT_ENTRY_FUEL_EXPEND_TAG_V1              : gb_bool_shape('Direct entry fuel expenditure'),
        ismvt.IS_COST_FUEL_PER_LITER_TAG_V1                   : gb_float_shape('Cost of fuel per liter'),
        ismvt.IS_ICT_EQUIPMENT_RECORDS_TAG_V1                 : gb_shape('IS ICT Equipment Records', '1D dict',
                                                                         dim=[is_ict_dim],      
                                                                         indicies=[is_ict_ids],
                                                                         innerShape={
                                                                                        'ictEquipID' : gb_str_shape('ICT equipment ID'),
                                                                                        'name' : gb_str_shape('Name'),
                                                                                        'currNumber' : gb_float_shape('Current number'),
                                                                                        'unitCost' : gb_float_shape('Unit cost'),
                                                                                        'workingLifeInYears' : gb_float_shape('Working life in years'),
                                                                                        'opCostPercCapCost' : gb_float_shape('Operating cost percent capital cost'),
                                                                                        'repairAndMaintPercCapCost' : gb_float_shape('Repair and maintenance percent capital cost'),
                                                                                        'targNumPerFacility' : gb_shape('Target number per facility', '1D float', dim=[is_facility_dim], indicies=[is_facility_ids]),
                                                                                        'targNumPerFacilityOtherNonFacBased' : gb_int_shape('Target number per facility other non-facility based'),
                                                                                        
                                                                                }
                                                                ),
        ismvt.IS_PROG_MGMT_DATA_TAG_V1                        : gb_shape('IS Program Management Data',  'dict',
                                                                           innerShape={
                                                                              'mainRowCosts': gb_shape('Main row costs', '2D float',
                                                                                dim=[is_pc_standard_rows_dim, gb_year_dim],
                                                                                indicies=[is_pc_standard_rows_id, gb_year_indices])
                                                                        },
                                                                           has_default=False
                                                                ),
        ismvt.IS_CALC_TOTAL_MED_EQUIP_COSTS_V1                : gb_bool_shape('Calculate total medical equipment costs flags'),
        ismvt.IS_CALC_MAINT_COSTS_AS_PERC_MED_EQUIP_COSTS_TAG : gb_bool_shape('Calculate maintenance costs as perc med equip costs flags'), # Enter directly/absolute number is default

        # # Outputs / results

        ismvt.IS_RES_TOTAL_NUM_FAC_AVAIL_TAG                     : gb_shape('Total number of facilities available', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_TOTAL_COST_BUILD_EQUIP_NEW_FAC_TAG          : gb_shape('Total cost to build and equip new facilities', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_NEW_FAC_CONSTRUCT_TAG                       : gb_shape('New facility construction costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_TOTAL_NUM_BEDS_AVAIL_TAG                    : gb_shape('Total number of beds available', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_NEW_FAC_FURNITURE_COSTS_TAG                 : gb_shape('New facility furniture costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_FAC_LAND_COSTS_TAG                          : gb_shape('Facility land costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_NEW_FAC_CONSTRUCT_COSTS_TAG                 : gb_shape('New facility construction costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_NEW_FAC_MED_EQUIP_COSTS_TAG                 : gb_shape('New facility medical equipment costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_FAC_REHAB_COSTS_TAG                         : gb_shape('Facility rehab costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_FAC_TOTAL_OP_COSTS_TAG                      : gb_shape('Facility total operating costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_OTHER_FAC_OP_COSTS_TAG                      : gb_shape('Other facility operating costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_FAC_WATER_COSTS_TAG                         : gb_shape('Facility water costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        ismvt.IS_RES_FAC_ELECTRIC_COSTS_TAG                      : gb_shape('Facility electric costs', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),
        
        ismvt.IS_RES_VEHIC_AVAIL_TAG                             : gb_shape('Vehicles available', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_PURCH_TAG                             : gb_shape('Vehicle purchase costs', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_PURCH_CAP_COSTS_TAG                   : gb_shape('Vehicle purchase capital costs', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_PURCH_CAP_COSTS_BY_FAC_TAG            : gb_shape('Vehicle purchase capital costs by facility', '2D float', contingent=True, dim=[is_vehicle_dim, is_facility_dim], indicies=[is_vehicle_ids, is_facility_ids]),
        ismvt.IS_RES_VEHIC_PURCH_CAP_COSTS_OTHER_NON_FAC_TAG     : gb_shape('Vehicle purchase capital costs other non-facility', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_MAINT_REPAIR_COSTS_TAG                : gb_shape('Vehicle maintenance and repair costs', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_MAINT_REPAIR_COSTS_BY_FAC_TAG         : gb_shape('Vehicle maintenance and repair costs by facility', '2D float', contingent=True, dim=[is_vehicle_dim, is_facility_dim], indicies=[is_vehicle_ids, is_facility_ids]),
        ismvt.IS_RES_VEHIC_MAINT_REPAIR_COSTS_OTHER_NON_FAC_TAG  : gb_shape('Vehicle maintenance and repair costs other non-facility', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_FUEL_COSTS_TAG                        : gb_shape('Vehicle fuel costs', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_FUEL_COSTS_BY_FAC_TAG                 : gb_shape('Vehicle fuel costs by facility', '2D float', contingent=True, dim=[is_vehicle_dim, is_facility_dim], indicies=[is_vehicle_ids, is_facility_ids]),
        ismvt.IS_RES_VEHIC_FUEL_COSTS_OTHER_NON_FAC_TAG          : gb_shape('Vehicle fuel costs other non-facility', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_DRIVER_SALARIES_TAG                   : gb_shape('Vehicle driver salaries', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        ismvt.IS_RES_VEHIC_DRIVER_SALARIES_BY_FAC_TAG            : gb_shape('Vehicle driver salaries by facility', '2D float', contingent=True, dim=[is_vehicle_dim, is_facility_dim], indicies=[is_vehicle_ids, is_facility_ids]),
        ismvt.IS_RES_VEHIC_DRIVER_SALARIES_OTHER_NON_FAC_TAG     : gb_shape('Vehicle driver salaries other non-facility', '2D float', contingent=True, dim=[is_vehicle_dim, gb_year_dim], indicies=[is_vehicle_ids, gb_year_indices]),
        
        ismvt.IS_RES_ICT_AVAIL_TAG                               : gb_shape('ICT available', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        ismvt.IS_RES_ICT_PURCH_TAG                               : gb_shape('ICT purchase costs', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        ismvt.IS_RES_ICT_CAP_COSTS_TAG                           : gb_shape('ICT capital costs', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        ismvt.IS_RES_ICT_CAP_COSTS_BY_FAC_TAG                    : gb_shape('ICT capital costs by facility', '2D float', contingent=True, dim=[is_ict_dim, is_facility_dim], indicies=[is_ict_ids, is_facility_ids]),
        ismvt.IS_RES_ICT_CAP_COSTS_OTHER_NON_FAC_TAG             : gb_shape('ICT capital costs other non-facility', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        ismvt.IS_RES_ICT_MAINT_REPAIR_OP_COSTS_TAG               : gb_shape('ICT maintenance and repair operating costs', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        ismvt.IS_RES_ICT_MAINT_REPAIR_OP_COSTS_BY_FAC_TAG        : gb_shape('ICT maintenance and repair operating costs by facility', '2D float', contingent=True, dim=[is_ict_dim, is_facility_dim], indicies=[is_ict_ids, is_facility_ids]),
        ismvt.IS_RES_ICT_MAINT_REPAIR_OP_COSTS_OTHER_NON_FAC_TAG : gb_shape('ICT maintenance and repair operating costs other non-facility', '2D float', contingent=True, dim=[is_ict_dim, gb_year_dim], indicies=[is_ict_ids, gb_year_indices]),
        
        ismvt.IS_RES_MED_EQUIP_MAINT_CALIB_COSTS_TB_TAG          : gb_shape('Medical equipment maintenance calibration costs by facility', '2D float', contingent=True, dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices]),

        # # TB costing mode
        ismvt.IS_RES_EQUIPMENT_COSTS_DSC_V1                      : gb_shape('Equipment costs by facility', '2D float', dim=[is_facility_dim, gb_year_dim], indicies=[is_facility_ids, gb_year_indices])
    
}


import numpy as np

def goveqs_basissmall(f_in, t, r, M, i, prm): #function [out, lam] = goveqs_basissmall(t, in, M, i, prm)             % <--- Arguments changed to accommodate prm

    invec = f_in[0:i['nstates']] #invec = in(1:i.nstates);
    #r['betared'] = np.array([r['betared']])
    fac = pow(r['betared'],max(t-2000, 0))#fac = pow(r['betared'][0],max(t-2000, 0));#fac = pow(r['betared'][0],max(t-2003, 0)); 
    
    #%try
    #% Normalise by populations
    lam = M['lambda']*invec/sum(invec)

    #What if t is less than 2015?
    #nlin_ds = lam[0]*M['nlin']['a1']['ds']+ lam[1]*M['nlin']['a2']['ds']+ lam[2]*M['nlin']['a3']['ds']+lam[3]*M['nlin']['a4']['ds']+ lam[4]*M['nlin']['a5']['ds']#nlin_ds = lam(1)*M.nlin.a1.ds+ lam(2)*M.nlin.a2.ds+ lam(3)*M.nlin.a3.ds+lam(4)*M.nlin.a4.ds+ lam(5)*M.nlin.a5.ds;
    nlin_ds = (lam[0]*M['nlin']['a1']['ds']+ lam[1]*M['nlin']['a2']['ds']+ lam[2]*M['nlin']['a3']['ds']+lam[3]*M['nlin']['a4']['ds']+ lam[4]*M['nlin']['a5']['ds'])#*pow(r['betared'][0],t-2015)#nlin_ds = lam(1)*M.nlin.a1.ds+ lam(2)*M.nlin.a2.ds+ lam(3)*M.nlin.a3.ds+lam(4)*M.nlin.a4.ds+ lam(5)*M.nlin.a5.ds;
    if i['nx']>60 : #i.nx>60  %length(gps.strains)>1
        nlin_mdr = lam[5]*M['nlin']['a1']['mdr']+ lam[6]*M['nlin']['a2']['mdr']+ lam[7]*M['nlin']['a3']['mdr'] +lam[8]*M['nlin']['a4']['mdr']+ lam[9]*M['nlin']['a5']['mdr']#nlin_mdr = lam(6)*M.nlin.a1.mdr+ lam(7)*M.nlin.a2.mdr+ lam(8)*M.nlin.a3.mdr +lam(9)*M.nlin.a4.mdr+ lam(10)*M.nlin.a5.mdr; 
    else:
        nlin_mdr = 0   
    #end
    #%allmat = M.lin + M.Dxlin + rHIV*M.linHIV + nlin_ds + nlin_mdr;
    #allmat = M['lin'] + M['Dxlin'] +  nlin_ds + nlin_mdr #allmat = M.lin + M.Dxlin +  nlin_ds + nlin_mdr;
    #%allmat = M.lin + M.Dxlin + M.linLTBI +rHIV*M.linHIV + nlin_ds + nlin_mdr;
    #New, 2024-04-02
    allmat = M['lin'] + M['Dxlin'] + fac*M['linLTBI'] +  nlin_ds + nlin_mdr

    out = allmat@invec #out = allmat*invec;
    #% catch
    #%    keyboard; 
    #% end

    #% Implement deaths
    morts = np.sum(M['mortvec'],1)*invec#    morts = sum(M.mortvec,2).*invec;
    out = out - morts #out = out - morts;

    #% Implement births
    births = sum(morts) + prm['growth'] #births = sum(morts) + prm.growth;
    out[i['U']['a1']['h0']['v0']] = out[i['U']['a1']['h0']['v0']]+births #out(i.U.a1.h0.v0) = out(i.U.a1.h0.v0)+births;

    #% % Get the auxiliaries
    #% out(i.aux.inc)     = agg.inc*(sel.inc.*allmat)*invec;
    #% out(i.aux.noti)    = agg.noti*(sel.noti.*allmat)*invec;
    #% out(i.aux.noti2)   = sum((sel.noti2.*allmat)*invec);
    #% out(i.aux.mort(1)) = sum(M.mortvec(:,2).*invec);
    #% out(i.aux.mort(2)) = sum(M.mortvec(:,3).*invec);
    #n_out = np.concatenate([out, lam])
    #Python does not allow returning a vector that does not have the same length as the initial condition. The solution here would be either 
    #to pass a vector as parameter and write lam in it or increase the size of the initial condition...
    return(out) 